"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getHasher = exports.digest = exports.digestBroswer = exports.generateSalt = void 0;
const error_1 = require("./error");
const generateSalt = (length) => {
    if (length <= 0) {
        throw new error_1.SDJWTException('Salt length must be greater than 0.');
    }
    const array = new Uint8Array(length);
    globalThis.crypto.getRandomValues(array);
    const salt = Array.from(array, (byte) => byte.toString(16).padStart(2, '0')).join('');
    return salt;
};
exports.generateSalt = generateSalt;
function digestBroswer(data, algorithm = 'SHA-256') {
    return __awaiter(this, void 0, void 0, function* () {
        const { subtle } = globalThis.crypto;
        const ec = new TextEncoder();
        const digest = yield subtle.digest(algorithm, ec.encode(data));
        return Array.from(new Uint8Array(digest))
            .map((b) => b.toString(16).padStart(2, '0'))
            .join('');
    });
}
exports.digestBroswer = digestBroswer;
function digest(data, algorithm = 'SHA-256') {
    return __awaiter(this, void 0, void 0, function* () {
        const { subtle } = globalThis.crypto;
        const ec = new TextEncoder();
        const digest = yield subtle.digest(algorithm, ec.encode(data));
        return Array.from(new Uint8Array(digest))
            .map((b) => b.toString(16).padStart(2, '0'))
            .join('');
    });
}
exports.digest = digest;
const getHasher = (algorithm = 'SHA-256') => {
    return (data) => digest(data, algorithm);
};
exports.getHasher = getHasher;
