"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getHasher = exports.digest = exports.generateSalt = void 0;
const error_1 = require("./error");
const generateSalt = (length) => {
    if (length <= 0) {
        throw new error_1.SDJWTException('Salt length must be greater than 0.');
    }
    const Crypto = require('node:crypto');
    const saltBytes = Crypto.randomBytes(length);
    const salt = saltBytes.toString('hex');
    return salt;
};
exports.generateSalt = generateSalt;
const digest = (data, algorithm = 'SHA-256') => __awaiter(void 0, void 0, void 0, function* () {
    const Crypto = require('node:crypto');
    const nodeAlg = toNodeCryptoAlg(algorithm);
    const hash = Crypto.createHash(nodeAlg);
    hash.update(data);
    return hash.digest('hex');
});
exports.digest = digest;
const getHasher = (algorithm = 'SHA-256') => {
    return (data) => (0, exports.digest)(data, algorithm);
};
exports.getHasher = getHasher;
const toNodeCryptoAlg = (hashAlg) => hashAlg.replace('-', '').toLowerCase();
