"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Disclosure = void 0;
const base64url_1 = require("./base64url");
const error_1 = require("./error");
class Disclosure {
    constructor(data) {
        if (data.length === 2) {
            this.salt = data[0];
            this.value = data[1];
            return;
        }
        if (data.length === 3) {
            this.salt = data[0];
            this.key = data[1];
            this.value = data[2];
            return;
        }
        throw new error_1.SDJWTException('Invalid disclosure data');
    }
    static fromEncode(s) {
        const item = JSON.parse(base64url_1.Base64Url.decode(s));
        return Disclosure.fromArray(item);
    }
    static fromArray(item) {
        return new Disclosure(item);
    }
    encode() {
        return base64url_1.Base64Url.encode(JSON.stringify(this.decode()));
    }
    decode() {
        return this.key
            ? [this.salt, this.key, this.value]
            : [this.salt, this.value];
    }
    digest(hasher) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this._digest) {
                const hash = yield hasher(this.encode());
                this._digest = base64url_1.Base64Url.encode(hash);
            }
            return this._digest;
        });
    }
}
exports.Disclosure = Disclosure;
