import { SDJwt } from './sdjwt';
import { DisclosureFrame, KBOptions, SDJWTCompact, SDJWTConfig, SignOptions, VerifyOptions, kbPayload } from './type';
import { KeyLike } from 'jose';
export * from './type';
export * from './sdjwt';
export * from './kbjwt';
export * from './crypto';
export * from './jwt';
export * from './base64url';
export * from './decoy';
export * from './disclosure';
export declare const defaultConfig: Required<SDJWTConfig>;
export declare class SDJwtInstance {
    static DEFAULT_ALG: string;
    static DEFAULT_HASH_ALG: string;
    private userConfig;
    constructor(userConfig?: SDJWTConfig);
    create(userConfig?: SDJWTConfig): SDJwtInstance;
    private isKBOptionsWithSigner;
    private createKBJwt;
    private SignJwt;
    private VerifyJwt;
    issue<Payload extends object>(payload: Payload, signOption?: SignOptions, disclosureFrame?: DisclosureFrame<Payload>, options?: {
        header?: object;
        sign_alg?: string;
        hash_alg?: string;
    }): Promise<SDJWTCompact>;
    present(encodedSDJwt: string, presentationKeys?: string[], options?: {
        kb?: KBOptions;
    }): Promise<SDJWTCompact>;
    verify(encodedSDJwt: string, verifyOption?: VerifyOptions, requiredClaimKeys?: string[], options?: {
        kb?: {
            publicKey: Uint8Array | KeyLike;
        };
    }): Promise<{
        payload: unknown;
        header: Record<string, any>;
        kb: {
            payload: kbPayload;
            header: import("./type").kbHeader;
        };
    } | {
        payload: unknown;
        header: Record<string, any>;
        kb?: undefined;
    }>;
    validate(encodedSDJwt: string, verifyOption?: VerifyOptions): Promise<{
        payload: unknown;
        header: Record<string, any>;
    }>;
    config(newConfig: SDJWTConfig): void;
    encode(sdJwt: SDJwt): SDJWTCompact;
    decode(endcodedSDJwt: SDJWTCompact): SDJwt<Record<string, any>, Record<string, any>, import("./type").kbHeader, kbPayload>;
    keys(endcodedSDJwt: SDJWTCompact): Promise<string[]>;
    presentableKeys(endcodedSDJwt: SDJWTCompact): Promise<string[]>;
    getClaims(endcodedSDJwt: SDJWTCompact): Promise<unknown>;
    getKeyBinding(endcodedSDJwt: SDJWTCompact): kbPayload | undefined;
}
declare const defaultInstance: SDJwtInstance;
export default defaultInstance;
