"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __exportStar = (this && this.__exportStar) || function(m, exports) {
    for (var p in m) if (p !== "default" && !Object.prototype.hasOwnProperty.call(exports, p)) __createBinding(exports, m, p);
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.SDJwtInstance = exports.defaultConfig = void 0;
const crypto_1 = require("./crypto");
const error_1 = require("./error");
const jwt_1 = require("./jwt");
const kbjwt_1 = require("./kbjwt");
const sdjwt_1 = require("./sdjwt");
const type_1 = require("./type");
__exportStar(require("./type"), exports);
__exportStar(require("./sdjwt"), exports);
__exportStar(require("./kbjwt"), exports);
__exportStar(require("./crypto"), exports);
__exportStar(require("./jwt"), exports);
__exportStar(require("./base64url"), exports);
__exportStar(require("./decoy"), exports);
__exportStar(require("./disclosure"), exports);
exports.defaultConfig = {
    omitTyp: false,
    hasher: crypto_1.digest,
    saltGenerator: crypto_1.generateSalt,
    signer: null,
    verifier: null,
};
class SDJwtInstance {
    constructor(userConfig) {
        this.userConfig = {};
        if (userConfig) {
            this.userConfig = userConfig;
        }
    }
    create(userConfig) {
        return new SDJwtInstance(userConfig);
    }
    isKBOptionsWithSigner(options) {
        if (options.signer) {
            return true;
        }
        return false;
    }
    createKBJwt(options) {
        return __awaiter(this, void 0, void 0, function* () {
            const { alg, payload } = options;
            const kbJwt = new kbjwt_1.KBJwt({
                header: {
                    typ: type_1.KB_JWT_TYP,
                    alg,
                },
                payload,
            });
            if (this.isKBOptionsWithSigner(options)) {
                const { signer } = options;
                yield kbJwt.signWithSigner(signer);
            }
            else {
                const { privateKey } = options;
                yield kbJwt.sign(privateKey);
            }
            return kbJwt;
        });
    }
    SignJwt(jwt, signOption) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!signOption) {
                const { signer } = this.userConfig;
                if (signer) {
                    yield jwt.signWithSigner(signer);
                    return jwt;
                }
                throw new error_1.SDJWTException('Invalid Sign Option');
            }
            if ('signer' in signOption) {
                yield jwt.signWithSigner(signOption.signer);
                return jwt;
            }
            yield jwt.sign(signOption.privateKey);
            return jwt;
        });
    }
    VerifyJwt(jwt, verifyOption) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!verifyOption) {
                const { verifier } = this.userConfig;
                if (verifier) {
                    return jwt.verifyWithVerifier(verifier);
                }
                throw new error_1.SDJWTException('Invalid Verify Option');
            }
            if ('verifier' in verifyOption) {
                return jwt.verifyWithVerifier(verifyOption.verifier);
            }
            return jwt.verify(verifyOption.publicKey);
        });
    }
    issue(payload, signOption, disclosureFrame, options) {
        var _a, _b, _c, _d, _e;
        return __awaiter(this, void 0, void 0, function* () {
            const haser = ((_a = this.userConfig.hasher) !== null && _a !== void 0 ? _a : options === null || options === void 0 ? void 0 : options.hash_alg)
                ? (0, crypto_1.getHasher)(options === null || options === void 0 ? void 0 : options.hash_alg)
                : exports.defaultConfig.hasher;
            const { packedClaims, disclosures } = yield (0, sdjwt_1.pack)(payload, disclosureFrame, haser, (_b = this.userConfig.saltGenerator) !== null && _b !== void 0 ? _b : exports.defaultConfig.saltGenerator);
            const alg = (_c = options === null || options === void 0 ? void 0 : options.sign_alg) !== null && _c !== void 0 ? _c : SDJwtInstance.DEFAULT_ALG;
            const OptionHeader = (_d = options === null || options === void 0 ? void 0 : options.header) !== null && _d !== void 0 ? _d : {};
            const CustomHeader = this.userConfig.omitTyp
                ? OptionHeader
                : Object.assign({ typ: type_1.SD_JWT_TYP }, OptionHeader);
            const header = Object.assign(Object.assign({}, CustomHeader), { alg });
            const jwt = new jwt_1.Jwt({
                header,
                payload: Object.assign(Object.assign({}, packedClaims), { _sd_alg: (_e = options === null || options === void 0 ? void 0 : options.hash_alg) !== null && _e !== void 0 ? _e : SDJwtInstance.DEFAULT_HASH_ALG }),
            });
            yield this.SignJwt(jwt, signOption);
            const sdJwt = new sdjwt_1.SDJwt({
                jwt,
                disclosures,
            });
            return sdJwt.encodeSDJwt();
        });
    }
    present(encodedSDJwt, presentationKeys, options) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!presentationKeys)
                return encodedSDJwt;
            const sdjwt = sdjwt_1.SDJwt.fromEncode(encodedSDJwt);
            const kbJwt = (options === null || options === void 0 ? void 0 : options.kb) ? yield this.createKBJwt(options.kb) : undefined;
            sdjwt.kbJwt = kbJwt;
            return sdjwt.present(presentationKeys.sort());
        });
    }
    verify(encodedSDJwt, verifyOption, requiredClaimKeys, options) {
        return __awaiter(this, void 0, void 0, function* () {
            const sdjwt = sdjwt_1.SDJwt.fromEncode(encodedSDJwt);
            if (!sdjwt.jwt) {
                throw new error_1.SDJWTException('Invalid SD JWT');
            }
            const { payload, header } = yield this.validate(encodedSDJwt, verifyOption);
            if (requiredClaimKeys) {
                const keys = yield sdjwt.keys();
                const missingKeys = requiredClaimKeys.filter((k) => !keys.includes(k));
                if (missingKeys.length > 0) {
                    throw new error_1.SDJWTException('Missing required claim keys: ' + missingKeys.join(', '));
                }
            }
            if (options === null || options === void 0 ? void 0 : options.kb) {
                if (!sdjwt.kbJwt) {
                    throw new error_1.SDJWTException('Key Binding JWT not exist');
                }
                const kb = yield sdjwt.kbJwt.verify(options.kb.publicKey);
                return { payload, header, kb };
            }
            return { payload, header };
        });
    }
    validate(encodedSDJwt, verifyOption) {
        return __awaiter(this, void 0, void 0, function* () {
            const sdjwt = sdjwt_1.SDJwt.fromEncode(encodedSDJwt);
            if (!sdjwt.jwt) {
                throw new error_1.SDJWTException('Invalid SD JWT');
            }
            const verifiedPayloads = yield this.VerifyJwt(sdjwt.jwt, verifyOption);
            const claims = yield sdjwt.getClaims();
            return { payload: claims, header: verifiedPayloads.header };
        });
    }
    config(newConfig) {
        this.userConfig = Object.assign(Object.assign({}, this.userConfig), newConfig);
    }
    encode(sdJwt) {
        return sdJwt.encodeSDJwt();
    }
    decode(endcodedSDJwt) {
        return sdjwt_1.SDJwt.fromEncode(endcodedSDJwt);
    }
    keys(endcodedSDJwt) {
        const sdjwt = sdjwt_1.SDJwt.fromEncode(endcodedSDJwt);
        return sdjwt.keys();
    }
    presentableKeys(endcodedSDJwt) {
        const sdjwt = sdjwt_1.SDJwt.fromEncode(endcodedSDJwt);
        return sdjwt.presentableKeys();
    }
    getClaims(endcodedSDJwt) {
        const sdjwt = sdjwt_1.SDJwt.fromEncode(endcodedSDJwt);
        return sdjwt.getClaims();
    }
    getKeyBinding(endcodedSDJwt) {
        var _a;
        const sdjwt = sdjwt_1.SDJwt.fromEncode(endcodedSDJwt);
        return (_a = sdjwt.kbJwt) === null || _a === void 0 ? void 0 : _a.payload;
    }
}
exports.SDJwtInstance = SDJwtInstance;
SDJwtInstance.DEFAULT_ALG = 'EdDSA';
SDJwtInstance.DEFAULT_HASH_ALG = 'sha-256';
const defaultInstance = new SDJwtInstance();
exports.default = defaultInstance;
