import * as jose from 'jose';
import { Base64urlString, Signer, Verifier } from './type';
export type JwtData<Header extends Record<string, any>, Payload extends Record<string, any>> = {
    header?: Header;
    payload?: Payload;
    signature?: Base64urlString;
};
export declare class Jwt<Header extends Record<string, any> = Record<string, any>, Payload extends Record<string, any> = Record<string, any>> {
    header?: Header;
    payload?: Payload;
    signature?: Base64urlString;
    constructor(data?: JwtData<Header, Payload>);
    static decodeJWT<Header extends Record<string, any> = Record<string, any>, Payload extends Record<string, any> = Record<string, any>>(jwt: string): {
        header: Header;
        payload: Payload;
        signature: Base64urlString;
    };
    static fromEncode<Header extends Record<string, any> = Record<string, any>, Payload extends Record<string, any> = Record<string, any>>(encodedJwt: string): Jwt<Header, Payload>;
    setHeader(header: Header): Jwt<Header, Payload>;
    setPayload(payload: Payload): Jwt<Header, Payload>;
    sign(privateKey: Uint8Array | jose.KeyLike): Promise<string>;
    signWithSigner(signer: Signer): Promise<string>;
    encodeJwt(): string;
    verify(publicKey: Uint8Array | jose.KeyLike): Promise<{
        payload: Payload;
        header: Header;
    }>;
    verifyWithVerifier(verifier: Verifier): Promise<{
        payload: Payload;
        header: Header;
    }>;
}
