"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Jwt = void 0;
const base64url_1 = require("./base64url");
const error_1 = require("./error");
const jose = __importStar(require("jose"));
class Jwt {
    constructor(data) {
        this.header = data === null || data === void 0 ? void 0 : data.header;
        this.payload = data === null || data === void 0 ? void 0 : data.payload;
        this.signature = data === null || data === void 0 ? void 0 : data.signature;
    }
    static decodeJWT(jwt) {
        const { 0: header, 1: payload, 2: signature, length } = jwt.split('.');
        if (length !== 3) {
            throw new error_1.SDJWTException('Invalid JWT as input');
        }
        return {
            header: JSON.parse(base64url_1.Base64Url.decode(header)),
            payload: JSON.parse(base64url_1.Base64Url.decode(payload)),
            signature: signature,
        };
    }
    static fromEncode(encodedJwt) {
        const { header, payload, signature } = Jwt.decodeJWT(encodedJwt);
        const jwt = new Jwt({
            header,
            payload,
            signature,
        });
        return jwt;
    }
    setHeader(header) {
        this.header = header;
        return this;
    }
    setPayload(payload) {
        this.payload = payload;
        return this;
    }
    sign(privateKey) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.header || !this.payload) {
                throw new error_1.SDJWTException('Sign Error: Invalid JWT');
            }
            // @ts-ignore
            const header = this.header;
            const encodedJwt = yield new jose.SignJWT(this.payload)
                .setProtectedHeader(header)
                .sign(privateKey);
            const { signature } = Jwt.decodeJWT(encodedJwt);
            this.signature = signature;
            return encodedJwt;
        });
    }
    signWithSigner(signer) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.header || !this.payload) {
                throw new error_1.SDJWTException('Sign Error: Invalid JWT');
            }
            const header = base64url_1.Base64Url.encode(JSON.stringify(this.header));
            const payload = base64url_1.Base64Url.encode(JSON.stringify(this.payload));
            const data = `${header}.${payload}`;
            this.signature = yield signer(data);
            return this.encodeJwt();
        });
    }
    encodeJwt() {
        if (!this.header || !this.payload || !this.signature) {
            throw new error_1.SDJWTException('Serialize Error: Invalid JWT');
        }
        const header = base64url_1.Base64Url.encode(JSON.stringify(this.header));
        const payload = base64url_1.Base64Url.encode(JSON.stringify(this.payload));
        const signature = this.signature;
        const compact = `${header}.${payload}.${signature}`;
        return compact;
    }
    verify(publicKey) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.header || !this.payload || !this.signature) {
                throw new error_1.SDJWTException('Verify Error: Invalid JWT');
            }
            const jwt = this.encodeJwt();
            try {
                yield jose.jwtVerify(jwt, publicKey);
            }
            catch (e) {
                throw new error_1.SDJWTException('Verify Error: Invalid JWT Signature');
            }
            return { payload: this.payload, header: this.header };
        });
    }
    verifyWithVerifier(verifier) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.header || !this.payload || !this.signature) {
                throw new error_1.SDJWTException('Verify Error: Invalid JWT');
            }
            const header = base64url_1.Base64Url.encode(JSON.stringify(this.header));
            const payload = base64url_1.Base64Url.encode(JSON.stringify(this.payload));
            const data = `${header}.${payload}`;
            const verified = verifier(data, this.signature);
            if (!verified) {
                throw new error_1.SDJWTException('Verify Error: Invalid JWT Signature');
            }
            return { payload: this.payload, header: this.header };
        });
    }
}
exports.Jwt = Jwt;
