import { Disclosure } from './disclosure';
import { Jwt } from './jwt';
import { KBJwt } from './kbjwt';
import { DisclosureFrame, Hasher, SDJWTCompact, SaltGenerator, kbHeader, kbPayload } from './type';
export type SDJwtData<Header extends Record<string, any>, Payload extends Record<string, any>, KBHeader extends kbHeader = kbHeader, KBPayload extends kbPayload = kbPayload> = {
    jwt?: Jwt<Header, Payload>;
    disclosures?: Array<Disclosure<any>>;
    kbJwt?: KBJwt<KBHeader, KBPayload>;
};
export declare class SDJwt<Header extends Record<string, any> = Record<string, any>, Payload extends Record<string, any> = Record<string, any>, KBHeader extends kbHeader = kbHeader, KBPayload extends kbPayload = kbPayload> {
    jwt?: Jwt<Header, Payload>;
    disclosures?: Array<Disclosure<any>>;
    kbJwt?: KBJwt<KBHeader, KBPayload>;
    constructor(data?: SDJwtData<Header, Payload, KBHeader, KBPayload>);
    static decodeSDJwt<Header extends Record<string, any> = Record<string, any>, Payload extends Record<string, any> = Record<string, any>, KBHeader extends kbHeader = kbHeader, KBPayload extends kbPayload = kbPayload>(sdjwt: SDJWTCompact): {
        jwt: Jwt<Header, Payload>;
        disclosures: Array<Disclosure<any>>;
        kbJwt?: KBJwt<KBHeader, KBPayload>;
    };
    static fromEncode<Header extends Record<string, any> = Record<string, any>, Payload extends Record<string, any> = Record<string, any>, KBHeader extends kbHeader = kbHeader, KBPayload extends kbPayload = kbPayload>(encodedSdJwt: SDJWTCompact): SDJwt<Header, Payload>;
    present(keys: string[]): Promise<SDJWTCompact>;
    encodeSDJwt(): SDJWTCompact;
    keys(): Promise<string[]>;
    presentableKeys(): Promise<string[]>;
    getClaims<T>(): Promise<T>;
}
export declare const listKeys: (obj: any, prefix?: string) => string[];
export declare const pack: <T extends object>(claims: T, disclosureFrame?: DisclosureFrame<T> | undefined, hasher?: Hasher, saltGenerator?: SaltGenerator) => Promise<{
    packedClaims: any;
    disclosures: Array<Disclosure<any>>;
}>;
export declare const unpackArray: (arr: Array<any>, map: Record<string, Disclosure<any>>, prefix?: string) => {
    unpackedObj: any;
    disclosureKeymap: Record<string, string>;
};
export declare const unpackObj: (obj: any, map: Record<string, Disclosure<any>>, prefix?: string) => {
    unpackedObj: any;
    disclosureKeymap: Record<string, string>;
};
export declare const createHashMapping: (disclosures: Array<Disclosure<any>>, hasher?: Hasher) => Promise<Record<string, Disclosure<any>>>;
export declare const unpack: (sdjwtPayload: any, disclosures: Array<Disclosure<any>>) => Promise<{
    unpackedObj: any;
    disclosureKeymap: Record<string, string>;
}>;
